@@ -1,4 +1,4 @@
 #!/bin/sh
-BASE=.tox/py36-dj21-pylint
+BASE=.tox/pylint-dj32-py38
 source $BASE/bin/activate
 DJANGO_SETTINGS_MODULE=salesforce.testrunner.settings $BASE/bin/pylint salesforce $@
