@@ -18,5 +18,8 @@ def complain(*args, **kwargs):
 
 
 class DatabaseClient(BaseDatabaseClient):
-    # pylint:disable=too-few-public-methods
     runshell = complain
+
+    @classmethod
+    def settings_to_cmd_args_env(cls, settings_dict, parameters):
+        complain()
