@@ -30,6 +30,14 @@ AliasMapItems = List[Tuple[
 ]]
 
 
+class SfParams:  # like an immutable DataClass: clone when updating
+    def __init__(self):
+        self.query_all = False
+        self.all_or_none = None  # type: Optional[bool]
+        self.edge_updates = False
+        self.minimal_aliases = False
+
+
 class SQLCompiler(sql_compiler.SQLCompiler):
     """
     A subclass of the default SQL compiler for the SOQL dialect.
@@ -38,8 +46,13 @@ class SQLCompiler(sql_compiler.SQLCompiler):
 
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
+        self.sf_params = SfParams()
         self.root_aliases = []  # type: List[str]
 
+    def set_sf_params(self, sf_params: SfParams) -> 'SQLCompiler':
+        self.sf_params = sf_params
+        return self
+
     def get_from_clause(self) -> Tuple[List[str], List[Any]]:
         """
         Return the FROM clause, converted the SOQL dialect.
@@ -86,7 +99,18 @@ class SQLCompiler(sql_compiler.SQLCompiler):
                 return sql_field
             pre, field, post = '', sql_field, ''
         tab_name, field_name = field.split('.')
-        ret = "%s%s.%s%s" % (pre, soql_trans[tab_name], field_name, post)
+        if self.sf_params.minimal_aliases:
+            assert len(self.root_aliases) == 1
+            if tab_name == self.root_aliases[0]:
+                trans_tab_name = ''
+            else:
+                trans_root = soql_trans[self.root_aliases[0]]
+                assert soql_trans[tab_name].startswith(trans_root + '.')
+                trans_tab_name = soql_trans[tab_name].replace(trans_root + '.', '', 1)
+        else:
+            trans_tab_name = soql_trans[tab_name]
+        dot = '.' if trans_tab_name else ''
+        ret = "%s%s%s%s%s" % (pre, trans_tab_name, dot, field_name, post)
         if debug_ >= 2:
             print('** sf_fix_field: {!r} -> {!r}'.format(sql_field, ret))
         return ret
@@ -292,7 +316,7 @@ class SQLCompiler(sql_compiler.SQLCompiler):
         if self.soql_trans is not None:
             return self.soql_trans
         if not _alias_map_items and not self.query.alias_map:
-            # empty alias_map is possible due to field expr in Django 1.8
+            # empty alias_map is possible due to field expr
             return {}
         # Unified interface:
         #   alias_map_items = [(lhs, table, join_cols_, rhs),...]
