@@ -9,6 +9,7 @@
 Salesforce introspection code.  (like django.db.backends.*.introspection)
 """
 
+import json
 import logging
 import re
 from collections import OrderedDict, namedtuple
@@ -45,6 +46,7 @@ PROBLEMATIC_OBJECTS = [
     'AsyncOperationStatus',  # new in API 46.0 Summer '19
     'FlowExecutionErrorEvent',  # new in API 47.0 Winter '20 - missing 'Id'
     'FlowOrchestrationEvent',  # new in API 53.0 Winter '22
+    'DataObjectDataChgEvent',  # new in API 55.0 Summer '22 (no 'Id' field)
 ]
 
 # this global variable is for `salesforce.management.commands.inspectdb`
@@ -148,6 +150,8 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
 
     def table_description_cache(self, table: str) -> Dict[str, Any]:
         if table not in self._table_description_cache:
+            if table == 'django_migrations':
+                raise ValueError("The internal table 'django_migrations' is not a normal Model.")
             log.debug('Request API URL: GET sobjects/%s/describe', table)
             response = self.connection.connection.handle_api_exceptions('GET', self.sobjects_prefix, table, 'describe/'
                                                                         )
@@ -198,13 +202,26 @@ class DatabaseIntrospection(BaseDatabaseIntrospection):
             # use symbolic names NOT_UPDATEABLE, NOT_CREATABLE, READ_ONLY instead of 1, 2, 3
             sf_read_only = (0 if field['updateable'] else 1) | (0 if field['createable'] else 2)
             params['sf_read_only'] = reverse_models_names[sf_read_only]
-        if field['defaultedOnCreate'] and field['createable']:
-            if field['defaultValue'] is None:
-                params['default'] = SymbolicModelsName('DEFAULTED_ON_CREATE')
+
+        if field['defaultValue'] is not None:
+            default: Optional[SymbolicModelsName] = field['defaultValue']
+        elif field['defaultValueFormula']:
+            if re.match(r'^(?:(?:-?[0-9]+(?:\.[0-9]+)?)|(?:"(?:[^"]|\")*"))$', field['defaultValueFormula']):
+                # (int, float, str)
+                default = json.loads(field['defaultValueFormula'])
+            elif field['defaultValueFormula'].lower() in ('true', 'false'):
+                # bool not important - probably not used
+                default = field['defaultValueFormula'].lower() == 'true'
             else:
-                params['default'] = SymbolicModelsName('DefaultedOnCreate', field['defaultValue'])
-        elif field['defaultValue'] is not None:
-            params['default'] = field['defaultValue']
+                default = None
+                params['ref_comment'] = 'Warning: not a simple defaultValueFormula'
+        else:
+            default = None
+        if default is not None:
+            params['default'] = default
+        elif field['defaultedOnCreate'] and field['createable']:
+            params['default'] = SymbolicModelsName('DEFAULTED_ON_CREATE')
+
         if field['inlineHelpText']:
             params['help_text'] = field['inlineHelpText']
         if field['picklistValues']:
