@@ -13,13 +13,14 @@ import typing  # pylint:disable=unused-import
 
 from django.conf import settings
 from django.db import NotSupportedError, models
+from django.db.models import constants
 from django.db.models import query as models_query, Model
 from django.db.models.sql import where as sql_where
 from django.db.utils import DEFAULT_DB_ALIAS
 import django
 
 from salesforce.backend.indep import get_sf_alt_pk
-from salesforce.backend import compiler, DJANGO_22_PLUS, DJANGO_30_PLUS, DJANGO_40_PLUS
+from salesforce.backend import compiler, DJANGO_22_PLUS, DJANGO_30_PLUS, DJANGO_40_PLUS, DJANGO_41_PLUS
 from salesforce.backend.models_sql_query import SalesforceQuery
 from salesforce.backend.operations import BULK_BATCH_SIZE
 from salesforce.router import is_sf_database
@@ -82,10 +83,15 @@ class SalesforceQuerySet(models_query.QuerySet, Generic[_T]):
     def simple_select_related(self, *fields: str) -> NoReturn:  # pylint:disable=no-self-use
         raise NotSupportedError("Obsoleted method .simple_select_related(), use .select_related() instead")
 
-    def bulk_create(self, objs: Iterable[_T], batch_size: Optional[int] = None, ignore_conflicts: bool = False
+    def bulk_create(self, objs: Iterable[_T], batch_size: Optional[int] = None,
+                    ignore_conflicts: bool = False,
+                    update_conflicts: bool = False,
+                    update_fields: Optional[List[str]] = None,
+                    unique_fields: Optional[List[str]] = None,
                     ) -> List[_T]:
         # parameter 'ignore_conflicts=False' is new in Django 2.2
         kwargs = {'ignore_conflicts': ignore_conflicts} if DJANGO_22_PLUS else {}
+        assert not update_conflicts and update_fields is None and unique_fields is None
         if getattr(self.model, '_salesforce_object', '') == 'extended' and not is_sf_database(self.db):
             objs = list(objs)
             for x in objs:
@@ -106,6 +112,7 @@ class SalesforceQuerySet(models_query.QuerySet, Generic[_T]):
            query_all: Optional[bool] = None,
            all_or_none: Optional[bool] = None,
            edge_updates: Optional[bool] = None,
+           minimal_aliases: Optional[bool] = None,
            ) -> 'SalesforceQuerySet[_T]':
         """Set additional parameters for queryset methods with Salesforce.
 
@@ -123,6 +130,7 @@ class SalesforceQuerySet(models_query.QuerySet, Generic[_T]):
             query_all=query_all,
             all_or_none=all_or_none,
             edge_updates=edge_updates,
+            minimal_aliases=minimal_aliases,
         )
         return clone
 
@@ -133,8 +141,22 @@ class SalesforceQuerySet(models_query.QuerySet, Generic[_T]):
         setattr(query, 'sf_params', self.query.sf_params)
 
     # original Django method '._insert(...)' patched by .patch_insert_query(...)
-    if DJANGO_30_PLUS:
-        def _insert(self, objs, fields, returning_fields=None, raw=False, using=None, ignore_conflicts=False):
+    if DJANGO_41_PLUS:
+        def _insert(self, objs, fields,
+                    returning_fields=None, raw=False, using=None, on_conflict=None,
+                    update_fields=None, unique_fields=None):
+            assert on_conflict is None or on_conflict == constants.OnConflict.IGNORE  # pylint:disable=no-member
+            assert update_fields is None and unique_fields is None
+            self._for_write = True
+            if using is None:
+                using = self.db
+            query = models.sql.InsertQuery(self.model, on_conflict=on_conflict)
+            self.patch_insert_query(query)  # patch
+            query.insert_values(fields, objs, raw=raw)
+            return query.get_compiler(using=using).execute_sql(returning_fields)
+    elif DJANGO_30_PLUS:
+        def _insert(self, objs, fields,  # type: ignore[misc] # pylint:disable=arguments-differ
+                    returning_fields=None, raw=False, using=None, ignore_conflicts=False):
             self._for_write = True
             if using is None:
                 using = self.db
@@ -143,8 +165,8 @@ class SalesforceQuerySet(models_query.QuerySet, Generic[_T]):
             query.insert_values(fields, objs, raw=raw)
             return query.get_compiler(using=using).execute_sql(returning_fields)
     elif DJANGO_22_PLUS:
-        def _insert(self, objs, fields, return_id=False, raw=False,  # type: ignore[misc] # noqa pylint:disable=arguments-differ
-                    using=None, ignore_conflicts=False):
+        def _insert(self, objs, fields,  # type: ignore[misc] # pylint:disable=arguments-differ
+                    return_id=False, raw=False, using=None, ignore_conflicts=False):
             """
             Insert a new record for the given model. This provides an interface to
             the InsertQuery class and is how Model.save() is implemented.
