@@ -1,25 +1,24 @@
 """
 CursorWrapper (like django.db.backends.utils)
 """
-import datetime
 import decimal
 import logging
 import warnings
-from typing import Any, Callable, Tuple, TypeVar, Union, overload
+from itertools import islice
+from typing import Any, Callable, Iterable, Iterator, List, Tuple, TypeVar, Union, overload
 
-import pytz
-from django.conf import settings
-from django.core.exceptions import ImproperlyConfigured
 from django.db import models, NotSupportedError
 from django.db.models.sql import subqueries, Query, RawQuery
 
 from salesforce.backend import DJANGO_30_PLUS
 from salesforce.dbapi.driver import (
     DatabaseError, InternalError, SalesforceWarning, merge_dict,
-    register_conversion, arg_to_json, SALESFORCE_DATETIME_FORMAT)
-from salesforce.fields import NOT_UPDATEABLE, NOT_CREATEABLE, SF_PK
+    register_conversion, arg_to_json)
+from salesforce.fields import NOT_UPDATEABLE, NOT_CREATEABLE
 
+V = TypeVar('V')
 if not DJANGO_30_PLUS:
+    # a "do nothing" stub for Django < 3.0, where is no decorator @async_unsafe
     F = TypeVar('F', bound=Callable)
     F2 = TypeVar('F2', bound=Callable)
 
@@ -40,88 +39,26 @@ if not DJANGO_30_PLUS:
             func = message
             message = 'You cannot call this from an async context - use a thread or sync_to_async.'
             return decorator(func)
-        else:
-            return decorator
+        return decorator
 else:
-    from django.utils.asyncio import async_unsafe  # type: ignore[import,no-redef]  # noqa
+
+    from django.utils.asyncio import (  # type: ignore[import,no-redef] # noqa pylint:disable=unused-import,ungrouped-imports
+        async_unsafe
+    )
 
 log = logging.getLogger(__name__)
 
-# pylint:disable=invalid-name
 
 DJANGO_DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S.%f-00:00'
 
 MIGRATIONS_QUERY_TO_BE_IGNORED = "SELECT django_migrations.app, django_migrations.name FROM django_migrations"
 
 
-def prep_for_deserialize_inner(model, record, init_list=None):
-    fields = dict()
-    for x in model._meta.fields:
-        if not x.primary_key and (not init_list or x.name in init_list):
-            if x.column.endswith('.Type'):
-                # Type of generic foreign key
-                simple_column, _ = x.column.split('.')
-                fields[x.name] = record[simple_column]['Type']
-            else:
-                # Normal fields
-                field_val = record[x.column]
-                # db_type = x.db_type(connection=connections[using])
-                if x.__class__.__name__ == 'DateTimeField' and field_val is not None:
-                    d = datetime.datetime.strptime(field_val, SALESFORCE_DATETIME_FORMAT)
-                    d = d.replace(tzinfo=pytz.utc)
-                    if settings.USE_TZ:
-                        fields[x.name] = d.strftime(DJANGO_DATETIME_FORMAT)
-                    else:
-                        tz = pytz.timezone(settings.TIME_ZONE)
-                        d = tz.normalize(d.astimezone(tz))
-                        fields[x.name] = d.strftime(DJANGO_DATETIME_FORMAT[:-6])
-                else:
-                    fields[x.name] = field_val
-    return fields
-
-
-def prep_for_deserialize(model, record, using, init_list=None):  # pylint:disable=unused-argument
-    """
-    Convert a record from SFDC (decoded JSON) to dict(model string, pk, fields)
-    If fixes fields of some types. If names of required fields `init_list `are
-    specified, then only these fields are processed.
-    """
-    # TODO the parameter 'using' is not currently important.
-    attribs = record.pop('attributes')  # NOQA pylint:disable=unused-variable
-
-    mod = model.__module__.split('.')
-    if hasattr(model._meta, 'app_label'):
-        app_label = getattr(model._meta, 'app_label')
-    elif mod[-1] == 'models':
-        app_label = mod[-2]
-    else:
-        raise ImproperlyConfigured("Can't discover the app_label for %s, you must specify it via model meta options.")
-
-    if len(record.keys()) == 1 and model._meta.db_table in record:
-        # this is for objects with ManyToManyField and OneToOneField
-        while len(record) == 1:
-            record = list(record.values())[0]
-            if record is None:
-                return None
-
-    fields = prep_for_deserialize_inner(model, record, init_list=init_list)
-
-    if init_list and set(init_list).difference(fields).difference([SF_PK]):
-        raise DatabaseError("Not found some expected fields")
-
-    return dict(
-        model='.'.join([app_label, model.__name__]),
-        pk=record.pop('Id'),
-        fields=fields,
-    )
-
-
 def extract_values(query):
     """
     Extract values from insert or update query.
     Supports bulk_create
     """
-    # pylint
     if isinstance(query, subqueries.UpdateQuery):
         row = query.values
         return extract_values_inner(row, query)
@@ -138,10 +75,11 @@ def extract_values_inner(row, query):
     fields = query.model._meta.fields
     for _, field in enumerate(fields):
         sf_read_only = getattr(field, 'sf_read_only', 0)
+        is_date_auto = getattr(field, 'auto_now', False) or getattr(field, 'auto_now_add', False)
         if field.get_internal_type() == 'AutoField':
             continue
         if isinstance(query, subqueries.UpdateQuery):
-            if (sf_read_only & NOT_UPDATEABLE) != 0:
+            if (sf_read_only & NOT_UPDATEABLE) != 0 or is_date_auto:
                 continue
             value_or_empty = [value for qfield, model, value in query.values if qfield.name == field.name]
             if value_or_empty:
@@ -152,7 +90,7 @@ def extract_values_inner(row, query):
                 continue
             if hasattr(value, 'default'):
                 warnings.warn(
-                    "The field '{}.{}' has not been saved again with DEFAULTED_ON_CREATE value. "
+                    "The field '{}.{}' has been saved again with DEFAULTED_ON_CREATE value. "
                     "It is better to set a real value to it or to refresh it from the database "
                     "or restrict updated fields explicitly by 'update_fields='."
                     .format(field.model._meta.object_name, field.name),
@@ -184,7 +122,7 @@ class CursorWrapper:
         """
         self.db = db
         self.query = None
-        self.session = db.sf_session
+        self.session = db.sf_session  # this creates a TCP connection if doesn't exist
         self.rowcount = None
         self.first_row = None
         self.lastrowid = None  # not moved to driver because INSERT is implemented here
@@ -290,9 +228,6 @@ class CursorWrapper:
             self.cursor.rowcount = 0
         self.rowcount = self.cursor.rowcount
 
-    def query_more(self, nextRecordsUrl: str):
-        return self.handle_api_exceptions('GET', nextRecordsUrl)
-
     def execute_insert(self, query):
         table = query.model._meta.db_table
         if table == 'django_migrations':
@@ -437,54 +372,6 @@ class CursorWrapper:
         ret = self.db.connection.composite_request(composite_data)
         self.rowcount = len([x for x in ret.json()['compositeResponse'] if x['httpStatusCode'] == 204])
 
-    # The following 3 methods (execute_ping, id_request, versions_request)
-    # can be renamed soon or moved.
-
-    def urls_request(self):
-        """Empty REST API request is useful after long inactivity before POST.
-
-        It ensures that the token will remain valid for at least half life time
-        of the new token. Otherwise it would be an awkward doubt if a timeout on
-        a lost connection is possible together with token expire in a post
-        request (insert).
-        """
-        ret = self.handle_api_exceptions('GET', '')
-        return ret.json()
-
-    def id_request(self):
-        """The Force.com Identity Service (return type dict of str)"""
-        # https://developer.salesforce.com/page/Digging_Deeper_into_OAuth_2.0_at_Salesforce.com?language=en&language=en#The_Force.com_Identity_Service
-
-        if 'id' in self.oauth:
-            url = self.oauth['id']
-        else:
-            # dynamic auth without 'id' parameter
-            url = self.urls_request()['identity']
-        ret = self.handle_api_exceptions('GET', url)  # TODO
-        return ret.json()
-
-    def versions_request(self):
-        """List Available REST API Versions"""
-        return self.handle_api_exceptions('GET', '', api_ver='').json()
-
-    def query_results(self, results):
-        while True:
-            for rec in results['records']:
-                if rec['attributes']['type'] == 'AggregateResult' and hasattr(self.query, 'annotation_select'):
-                    annotation_select = self.query.annotation_select
-                    assert len(rec) - 1 == len(list(annotation_select.items()))
-                    # The 'attributes' info is unexpected for Django within fields.
-                    rec = [rec[k] for k, _ in annotation_select.items()]
-                yield rec
-
-            if results['done']:
-                break
-
-            # see about Retrieving the Remaining SOQL Query Results
-            # http://www.salesforce.com/us/developer/docs/api_rest/Content/dome_query.htm#retrieve_remaining_results_title
-            response = self.query_more(results['nextRecordsUrl'])
-            results = response.json(parse_float=decimal.Decimal)
-
     def __iter__(self):
         return self.cursor
 
@@ -514,9 +401,24 @@ class CursorWrapper:
         return self.cursor.handle_api_exceptions(method, *url_parts, **kwargs)
 
 
+def chunked(iterable: Iterable[V], n: int) -> Iterator[List[V]]:
+    """
+    Break an iterable into lists of a given length::
+
+    >>> assert list(chunked([1, 2, 3, 4, 5], 3)) == [[1, 2, 3], [4,5]]
+    """
+    iterable = iter(iterable)
+    while True:
+        chunk = list(islice(iterable, n))
+        if not chunk:
+            return
+        yield chunk
+
+
 def sobj_id(obj):
     assert obj._salesforce_object  # pylint:disable=protected-access
     return obj.pk
 
 
+# this JSON conversion is important for QuerySet.update(foreign_key_field=some_object)
 register_conversion(models.Model, json_conv=sobj_id, subclass=True)
