@@ -21,7 +21,7 @@ import re
 import pytz
 from salesforce.dbapi.exceptions import ProgrammingError
 
-_TRow = TypeVar('_TRow', List[Any], Dict[str, Any])
+_TRow = TypeVar('_TRow', Tuple[Any, ...], List[Any], Dict[str, Any])
 _T = TypeVar('_T')
 
 
@@ -62,7 +62,7 @@ class QQuery:
     # type of query: QRootQuery, QFieldSubquery, QWhereSubquery
     # type of field: QField, QAggregation, QFieldSubquery
 
-    # pylint:disable=too-few-public-methods,too-many-instance-attributes
+    # pylint:disable=too-many-instance-attributes
 
     def __init__(self, soql: Optional[str] = None) -> None:
         self.soql = None                  # type: Optional[str]
@@ -161,18 +161,15 @@ class QQuery:
         return out
 
     @overload                   # noqa
-    def parse_rest_response(self, records: Iterable[Dict[str, Any]], rowcount: int, row_type: Type[Dict[Any, Any]]
-                            ) -> Iterable[Dict[str, Any]]: ...
-    @overload                   # noqa
-    def parse_rest_response(self, records: Iterable[Dict[str, Any]], rowcount: int, row_type: Type[List[Any]]
-                            ) -> Iterable[List[Any]]: ...
+    def parse_rest_response(self, records: Iterable[Dict[str, Any]], rowcount: int, row_type: Type[_TRow]
+                            ) -> Iterable[_TRow]: ...
     @overload                   # noqa
     def parse_rest_response(self, records: Iterable[Dict[str, Any]], rowcount: int,
-                            ) -> Iterable[Dict[str, Any]]: ...
+                            ) -> Iterable[Tuple[Any, ...]]: ...
     def parse_rest_response(self, records: Iterable[Dict[str, Any]], # type: ignore[no-untyped-def] # noqa
-                            rowcount: int, row_type=list) -> Iterable[Any]:
+                            rowcount: int, row_type=tuple) -> Iterable[Any]:
         """Parse the REST API response to DB API cursor flat response"""
-        assert row_type in (dict, list)
+        assert row_type in (dict, list, tuple)
         if self.is_plain_count:
             # result of "SELECT COUNT() FROM ... WHERE ..."
             assert list(records) == []
@@ -180,9 +177,13 @@ class QQuery:
                 yield {'count': rowcount}
             elif issubclass(row_type, list):
                 yield [rowcount]  # originally [resp.json()['totalSize']]
+            elif issubclass(row_type, tuple):
+                yield (rowcount,)
         else:
-            while True:
-                for row_deep in records:
+            for row_deep in records:
+                if list(row_deep.keys()) == ['explain']:
+                    yield row_deep['explain']
+                else:
                     assert self.is_aggregation == (row_deep['attributes']['type'] == 'AggregateResult')
                     row_flat = self._make_flat(row_deep, path=(), subroots=self.subroots)
                     # TODO Will be the expression "or x['done']" really correct also for long subrequests?
@@ -191,13 +192,8 @@ class QQuery:
                         yield {k: fix_data_type(row_flat[k.lower()]) for k in self.aliases}
                     elif issubclass(row_type, list):
                         yield [fix_data_type(row_flat[k.lower()]) for k in self.aliases]
-                # if not resp['done']:
-                #     if not cursor:
-                #         raise ProgrammingError("Must get a cursor")
-                #     resp = cursor.query_more(resp['nextRecordsUrl']).json()
-                # else:
-                #     break
-                break
+                    elif issubclass(row_type, tuple):
+                        yield tuple(fix_data_type(row_flat[k.lower()]) for k in self.aliases)
 
 
 SALESFORCE_DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%f+0000'
