@@ -5,10 +5,10 @@
 # See LICENSE.md for details
 #
 
-from django.conf.urls import url
+from django.urls import path
 from . import views
 
 urlpatterns = [
-    url(r'^$', views.list_accounts, name='list_accounts'),
-    url(r'^search/$', views.search_accounts, name='search_accounts'),
+    path(r'', views.list_accounts, name='list_accounts'),
+    path(r'search/', views.search_accounts, name='search_accounts'),
 ]
