@@ -8,7 +8,7 @@ ADMINS = (
     # ('Your Name', 'your_email@example.com'),
 )
 
-PERSON_ACCOUNT_ACTIVATED = False
+SF_EXAMPLE_PERSON_ACCOUNT_ACTIVATED = False
 
 DATABASES = {
     'default': {
@@ -23,7 +23,7 @@ DATABASES = {
         "CONSUMER_SECRET": os.environ.get('SF_CONSUMER_SECRET', ''),
         'USER': os.environ.get('SF_USER', ''),
         'PASSWORD': os.environ.get('SF_PASSWORD', ''),
-        'HOST': 'https://login.salesforce.com',
+        'HOST': os.environ.get('SF_HOST', 'https://login.salesforce.com'),
         'TEST': {
             'DEPENDENCIES': [],
             'MIGRATE': False,   # to run tests without migrations in Django 3.1+
@@ -88,7 +88,7 @@ STATICFILES_FINDERS = (
 )
 
 # Make this unique, and don't share it with anybody.
-SECRET_KEY = '6$y&o(28l)#o1_2rafojb_&zxi*jnivkv)ygj#!01kt0ypsxe$'
+SECRET_KEY = '6$y&o(28l)#o1_2rafojb_&zxi*jnivkv)ygj#!01kt0ypsxe$'  # used only in tests
 
 SITE_ID = 1
 
@@ -186,18 +186,14 @@ LOGGING = {
     }
 }
 
-# Preventive workaround for some problems with IPv6 by restricting DNS queries
-# in the Python process only to IPv4, until the support by SFDC become stable.
-# SFDC enabled IPv6 for a week in March 2014. It caused long delays somewhere.
-# IPV4_ONLY = True
-
 # Name of primary key - by default 'id'. The value 'Id' was the default for
 # version "django-salesforce < 0.5".
+# The value SF_PK can not be changed after any migration for Salesforce has been created
 # SF_PK = 'Id'
 
 DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'  # not important for Salesforce, but for Django warnings
 
 try:
-    from salesforce.testrunner.local_settings import *  # NOQA pylint:disable=unused-wildcard-import,wildcard-import
+    from salesforce.testrunner.local_settings import *  # noqa pylint:disable=unused-wildcard-import,wildcard-import
 except ImportError:
     pass
