@@ -5,4 +5,4 @@ INSTALLED_APPS = [x for x in INSTALLED_APPS if not x.startswith('salesforce.test
 # INSTALLED_APPS += ('tests.inspectdb', 'tests.inspectdb.dependent_model')
 INSTALLED_APPS += ['tests.inspectdb.dependent_model.AutoModelConf',
                    'tests.inspectdb.dependent_model.DependentModelConf']
-ROOT_URLCONF = None
+ROOT_URLCONF = 'tests.no_urls'
