@@ -1,6 +1,6 @@
 """Backward compatible behaviour with primary key 'Id'."""
 from django.test import TestCase
-from tests.test_compatibility.models import Lead
+from tests.test_compatibility.models import Lead, B, AtoB
 from salesforce.backend.test_helpers import current_user, uid_version as uid
 
 
@@ -30,3 +30,20 @@ class DjangoCompatibility(TestCase):
         self.assertEqual(primary_key.auto_created, True)
         self.assertEqual(primary_key.get_internal_type(), 'AutoField')
         self.assertIn(primary_key.name, ('id', 'Id'))
+
+
+class ManyToManyTest(TestCase):
+
+    def test_subquery_filter_on_child(self):
+        """Filter with a Subquery() on a child object.
+
+        Especially useful with ManyToMany field relationships.
+        """
+        assoc = AtoB.objects.filter(a__email='a@example.com')
+        qs = B.objects.filter(pk__in=assoc.values('b'))
+        soql = (
+            "SELECT B__c.Id FROM B__c WHERE B__c.Id IN ("
+            "SELECT AtoB__c.B__c FROM AtoB__c WHERE AtoB__c.A__r.Email__c = 'a@example.com'"
+            ")"
+        )
+        self.assertEqual(str(qs.query), soql)
