@@ -1,9 +1,10 @@
 from django.utils.crypto import get_random_string
 from salesforce.testrunner.settings import DATABASES  # noqa # necessary for setup
 
-SECRET_KEY = get_random_string()
+SECRET_KEY = get_random_string(length=32)
 
 INSTALLED_APPS = ['tests.test_managers']
 DATABASE_ROUTERS = ["salesforce.router.ModelRouter"]
 SF_LAZY_CONNECT = True
 USE_TZ = True
+DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'  # not important for Salesforce, but for Django warnings
