@@ -3,4 +3,4 @@ from salesforce.testrunner.settings import INSTALLED_APPS
 
 INSTALLED_APPS = [x for x in INSTALLED_APPS if x != 'salesforce.testrunner.example']
 INSTALLED_APPS += ['tests.test_mixin']
-ROOT_URLCONF = None
+ROOT_URLCONF = 'tests.no_urls'
