@@ -80,7 +80,12 @@ def run():
         # in API 51.0 Spring '21
         'EmployeeUserSettings',
         # in API 52.0 Summer '21
-        'EinsteinAgentSettings'
+        'EinsteinAgentSettings',
+        # in API 54.0 Spring '22
+        'EinsteinDealInsightsSettings',
+        # in API 55.0 Summer '22
+        'IndustriesAutomotiveSettings', 'RelatedListColumnDefinition', 'RelatedListDefinition',
+        'SubscriptionManagementSettings', 'WarrantyLifecycleMgmtSettings',
     }
     problematic_write = {
         # any SalesforceError
